@@ -13,11 +13,9 @@ class TapiocaInstantiator:
         self.adapter_class = adapter_class
 
     def __call__(self, serializer_class=None, session=None, **kwargs):
-        refresh_token_default = kwargs.pop("refresh_token_by_default", False)
         return TapiocaClient(
             self.adapter_class(serializer_class=serializer_class),
             api_params=kwargs,
-            refresh_token_by_default=refresh_token_default,
             session=session,
         )
 
@@ -31,7 +29,6 @@ class TapiocaClient:
         request_kwargs=None,
         api_params=None,
         resource=None,
-        refresh_token_by_default=False,
         refresh_data=None,
         session=None,
         *args,
@@ -43,7 +40,6 @@ class TapiocaClient:
         self._api_params = api_params or {}
         self._request_kwargs = request_kwargs
         self._resource = resource
-        self._refresh_token_default = refresh_token_by_default
         self._refresh_data = refresh_data
         self._session = session
 
@@ -63,45 +59,22 @@ class TapiocaClient:
         return self._api.__class__(serializer_class=serializer_class)
 
     def _wrap_in_tapioca(self, data, *args, **kwargs):
-        return TapiocaClient(
-            self._instatiate_api(),
-            data=data,
-            api_params=self._api_params,
-            request_kwargs=kwargs.pop("request_kwargs", self._request_kwargs),
-            response=kwargs.pop("response", self._response),
-            resource=kwargs.pop("resource", self._resource),
-            refresh_token_by_default=self._refresh_token_default,
-            refresh_data=self._refresh_data,
-            session=self._session,
-            *args,
-            **kwargs
-        )
+        context = self._context(**{**kwargs, "data": data})
+        return TapiocaClient(*args, **context)
 
     def _wrap_in_tapioca_executor(self, data, *args, **kwargs):
-        return TapiocaClientExecutor(
-            self._instatiate_api(),
-            data=data,
-            api_params=self._api_params,
-            request_kwargs=kwargs.pop("request_kwargs", self._request_kwargs),
-            response=kwargs.pop("response", self._response),
-            resource=kwargs.pop("resource", self._resource),
-            refresh_token_by_default=self._refresh_token_default,
-            refresh_data=self._refresh_data,
-            session=self._session,
-            *args,
-            **kwargs
-        )
+        context = self._context(**{**kwargs, "data": data})
+        return TapiocaClientExecutor(*args, **context)
 
     def _context(self, **kwargs):
         context = dict(
             client=self,
-            api=self._api,
+            api=self._instatiate_api(),
             data=self._data,
-            response=self._response,
             api_params=self._api_params,
             request_kwargs=self._request_kwargs,
+            response=self._response,
             resource=self._resource,
-            refresh_token_default=self._refresh_token_default,
             refresh_data=self._refresh_data,
             session=self._session,
         )
@@ -331,7 +304,9 @@ class TapiocaClientExecutor(TapiocaClient):
             )
 
             should_refresh_token = (
-                refresh_token is not False and self._refresh_token_default
+                self._api.refresh_token is True
+                or self._api_params.get("refresh_token") is True
+                or refresh_token is True
             )
             auth_expired = await self._coro_wrap(
                 self._api.is_authentication_expired, tapioca_exception, **context
@@ -376,7 +351,7 @@ class TapiocaClientExecutor(TapiocaClient):
     async def _send(self, request_method, *args, **kwargs):
 
         if "semaphore" in kwargs and "semaphore_class" not in kwargs:
-            kwargs["semaphore_class"] = asyncio.Semaphore(kwargs.get("semaphore", 10))
+            kwargs["semaphore_class"] = asyncio.Semaphore(kwargs.pop("semaphore", 10))
 
         debug = kwargs.pop("debug") if "debug" in kwargs else False
         semaphore = (
@@ -400,7 +375,7 @@ class TapiocaClientExecutor(TapiocaClient):
 
         data = kwargs.pop("data") if "data" in kwargs else []
 
-        kwargs["semaphore_class"] = asyncio.Semaphore(kwargs.get("semaphore", 10))
+        kwargs["semaphore_class"] = asyncio.Semaphore(kwargs.pop("semaphore", 10))
 
         results = await asyncio.gather(
             *[
